@@ -16,12 +16,13 @@ def run_change_passphrase(
     remote_path,
 ):
     '''
-    Run the "key change-passprhase" action for the given repository.
+    Run the "key change-passphrase" action for the given repository.
     '''
     if (
         change_passphrase_arguments.repository is None
         or borgmatic.config.validate.repositories_match(
-            repository, change_passphrase_arguments.repository
+            repository,
+            change_passphrase_arguments.repository,
         )
     ):
         logger.info('Changing repository passphrase')
