@@ -7,13 +7,19 @@ import borgmatic.hooks.credential.parse
 logger = logging.getLogger(__name__)
 
 
+TIMEOUT_SECONDS = 10
+
+
 def initialize_monitor(
-    ping_url, config, config_filename, monitoring_log_level, dry_run
+    ping_url,
+    config,
+    config_filename,
+    monitoring_log_level,
+    dry_run,
 ):  # pragma: no cover
     '''
     No initialization is necessary for this monitor.
     '''
-    pass
 
 
 def ping_monitor(hook_config, config, config_filename, state, monitoring_log_level, dry_run):
@@ -43,6 +49,7 @@ def ping_monitor(hook_config, config, config_filename, state, monitoring_log_lev
         logger.debug(f'Using Ntfy ping URL {base_url}/{topic}')
 
         headers = {
+            'User-Agent': 'borgmatic',
             'X-Title': state_config.get('title'),
             'X-Message': state_config.get('message'),
             'X-Priority': state_config.get('priority'),
@@ -51,13 +58,16 @@ def ping_monitor(hook_config, config, config_filename, state, monitoring_log_lev
 
         try:
             username = borgmatic.hooks.credential.parse.resolve_credential(
-                hook_config.get('username')
+                hook_config.get('username'),
+                config,
             )
             password = borgmatic.hooks.credential.parse.resolve_credential(
-                hook_config.get('password')
+                hook_config.get('password'),
+                config,
             )
             access_token = borgmatic.hooks.credential.parse.resolve_credential(
-                hook_config.get('access_token')
+                hook_config.get('access_token'),
+                config,
             )
         except ValueError as error:
             logger.warning(f'Ntfy credential error: {error}')
@@ -68,8 +78,9 @@ def ping_monitor(hook_config, config, config_filename, state, monitoring_log_lev
         if access_token is not None:
             if username or password:
                 logger.warning(
-                    'ntfy access_token is set but so is username/password, only using access_token'
+                    'ntfy access_token is set but so is username/password, only using access_token',
                 )
+
             auth = requests.auth.HTTPBasicAuth('', access_token)
         elif (username and password) is not None:
             auth = requests.auth.HTTPBasicAuth(username, password)
@@ -82,7 +93,12 @@ def ping_monitor(hook_config, config, config_filename, state, monitoring_log_lev
         if not dry_run:
             logging.getLogger('urllib3').setLevel(logging.ERROR)
             try:
-                response = requests.post(f'{base_url}/{topic}', headers=headers, auth=auth)
+                response = requests.post(
+                    f'{base_url}/{topic}',
+                    auth=auth,
+                    timeout=TIMEOUT_SECONDS,
+                    headers=headers,
+                )
                 if not response.ok:
                     response.raise_for_status()
             except requests.exceptions.RequestException as error:
@@ -93,4 +109,3 @@ def destroy_monitor(ping_url_or_uuid, config, monitoring_log_level, dry_run):  #
     '''
     No destruction is necessary for this monitor.
     '''
-    pass
